@@ -1,16 +1,25 @@
+"""Define various classes to store timeseries and similar objects."""
 import numpy as np
 from scipy.integrate import trapz, cumtrapz
 import matplotlib.pyplot as plt
-from .singledof import Sdof, SdofNL
+# import matplotlib as mpl
+from .singledof import Sdof  # , SdofNL
 from scipy.fftpack import fft, fftfreq
 
+np.set_printoptions(threshold=50)
+
+
 class TimeSeries:
     """
-    TimeSeries object class. Defines time series
+    TimeSeries class. Defines time series. TimeSeries is a basic data type in the module.
+
+    Most of the calculations need a timeseries to be defined as a TimeSeries object.
     """
 
     def __init__(self, t, y):
         """
+        Define TimeSeries object.
+
         Parameters
         ----------
         t: (scalar or 1D array) time step (if scalar) or time axis (if 1D array)
@@ -20,8 +29,13 @@ class TimeSeries:
             t = np.arange(t, len(y) * t + 0.1 * t, t)
         self.t = t
         self.y = y
-        self.npts = len(t)
-        self.dt = t[1] - t[0]
+        self.npts = len(self.t)
+        self.dt = self.t[1] - self.t[0]
+        self.component = " "
+        self.duration = self.t[-1]
+        self.eqDate = " "
+        self.eqName = " "
+        self.filepath = " "
 
     def __repr__(self):
         a = ""
@@ -30,58 +44,57 @@ class TimeSeries:
         return a
 
     def set_tunit(self, unit):
-        """
-        Set unit for T(time) coordinates. Unit should be a string.
-        """
+        """Set unit for T(time) coordinates. Unit should be a string."""
         self.tunit = unit
 
     def set_yunit(self, unit):
-        """
-        Set unit for y coordinates. Unit should be a string.
-        """
+        """Set unit for y coordinates. Unit should be a string."""
         self.yunit = unit
 
     def set_t(self, coords):
-        """
-        Set T(time) coordinates. Should be a list or numpy array (1D)
-        """
+        """Set T(time) coordinates. Should be a list or numpy array (1D)."""
         self.t = coords
 
     def set_eqname(self, name):
-        """Set earthquake name"""
+        """Set earthquake name."""
         self.eqName = name
 
     def set_eqdate(self, date):
-        """Set earthquake date"""
+        """Set earthquake date."""
         self.eqDate = date
 
     def set_station(self, station):
-        """Recording station"""
+        """Set recording station."""
         self.station = station
 
     def set_component(self, comp):
-        """Directional component of record"""
+        """Directional component of record."""
         self.component = comp
 
     def set_dt(self, dt):
-        """Time step between data points"""
+        """Time step between data points."""
         self.dt = dt
 
     def set_npts(self, npts):
-        """Total number of points in the record"""
+        """Total number of points in the record."""
         self.npts = npts
 
     def set_duration(self, duration):
-        """Total duration of the record in seconds"""
+        """Total duration of the record in seconds."""
         self.duration = duration
 
     def set_filepath(self, filepath):
-        """Record filepath"""
+        """Set record filepath."""
         self.filepath = filepath
 
     def plot(self, log=False, **kwargs):
         """
-        Method plots time history of timeseries object. It accepts all the arguments that matplotlib.pyplot.subplots recognizes.
+        Plot time history of timeseries object.
+
+        It accepts all the arguments that matplotlib.pyplot.subplots recognize.
+        Parameters
+        ----------
+        log (boolean): Use log scale for plotting (default *False*).
 
         Returns
         -------
@@ -89,22 +102,25 @@ class TimeSeries:
 
         """
         fig, ax = plt.subplots(**kwargs)
-        ax.plot(self.t, self.y, color="black", linewidth=0.5)
+        ax.plot(self.t, self.y, color='k', label=str(self.eqName) + '_' + str(self.component))
         if hasattr(self, "tunit"):
             ax.set_xlabel("t ({})".format(self.tunit))
         if hasattr(self, "yunit"):
             ax.set_ylabel(str(self.yunit))
         if log:
             ax.set_xscale("log")
+        ax.set_xlim(left=0)
+        ax.legend()
         # plt.show()
         return fig
 
     def get_y(self, t):
+        """Perform 1D interpolation."""
         return np.interp(t, self.t, self.y)
 
-    def get_response_spectra(self, T=np.arange(0.1, 20.01, 1.0), xi=0.05):
+    def get_response_spectra_frequency_domain(self, T=np.arange(0.1, 100.001, 0.1), xi=0.05):
         """
-        Calculates linear elastic response spectra associated with the timeseries.
+        Calculate linear elastic response spectra associated with the timeseries.
 
         Parameters
         ----------
@@ -121,19 +137,29 @@ class TimeSeries:
         Sa = np.empty(specLength)
         for i in range(specLength):
             s = Sdof(T=T[i], xi=xi)
-            r = s.get_response(self, tsType="baseExcitation")
-            D = np.max(np.abs(r.y[0]))
-            V = np.max(np.abs(r.y[1]))
-            A = np.max(np.abs(r.acc))
-            Sd[i] = D
-            Sv[i] = V
-            Sa[i] = A
-        return ResponseSpectrum(T, Sd, Sv, Sa)
+            t, d, v, a = s.get_response_frequency_domain(self, tsType="baseExcitation")
+            Sd[i] = np.max(np.abs(np.real(d)))
+            Sv[i] = np.max(np.abs(np.real(v)))
+            Sa[i] = np.max(np.abs(np.real(a)))
+        return ResponseSpectra(T, Sd, Sv, Sa)
+
+    def get_response_spectra(self, T=np.arange(0.1, 100.001, 0.1), xi=0.05):
+        """
+        Calculate linear elastic response spectra associated with the timeseries.
+
+        Parameters
+        ----------
+        T: (1D array of floats) Periods corresponding to spectrum width
+        xi: (float) damping ratio
+
+        Returns
+        -------
+        ResponseSpectrum object with T, Sd, Sv, Sa as attributes.
+        """
+        return self.get_response_spectra_frequency_domain(T=T, xi=xi)
 
     def get_fourier_spectrum(self):
-        """
-        Computes fourier spectrum associated with the time series
-        """
+        """Compute fourier spectrum associated with the time series."""
         N = self.npts
         T = self.dt  # sampling interval
         yf = fft(self.y)
@@ -142,19 +168,21 @@ class TimeSeries:
         return FourierSpectrum(freq, yf, N)
 
     def get_power_spectrum(self):
-        """
-        Computes power spectrum associated with the time series
-        """
-        N = self.npts
+        """Compute power spectrum associated with the time series."""
+        dt = self.dt
         fourier_spectrum = self.get_fourier_spectrum()
         freq = fourier_spectrum.frequencies
-        df = freq[1] - freq[0]
-        powerAmp = 2.0 / df * fourier_spectrum.amplitude**2
-        return PowerSpectrum(freq, powerAmp, N)
+        # Power amplitude is multiplied by 2 to consider power from positive and negative frequencies
+        # The frequency 0 and nyquist apprear only once so divided later by 2.
+        powerAmp = 2 * dt / self.npts * np.abs(fourier_spectrum.amplitude)**2
+        powerAmp[0] = powerAmp[0]/2
+        idx = np.argmax(powerAmp)
+        powerAmp[idx] = powerAmp[idx]/2
+        return PowerSpectrum(freq, powerAmp, self.npts)
 
     def get_mean_period(self):
         """
-        Computes the simplified frequency content characterisation parameter according to  Rathje et al. [1998]
+        Compute the simplified frequency content characterisation parameter according to  Rathje et al. [1998].
 
         Returns
         -------
@@ -165,7 +193,7 @@ class TimeSeries:
         fourier_spectrum = self.get_fourier_spectrum()
         freq = fourier_spectrum.frequencies
         FAmp = fourier_spectrum.amplitude
-        boolArr = (freq > 0.25) & (freq < 20)
+        boolArr = (freq >= 0.25) & (freq <= 20)
         n = FAmp[boolArr] ** 2 / freq[boolArr]
         n = n.sum()
         d = FAmp[boolArr] ** 2
@@ -174,7 +202,7 @@ class TimeSeries:
 
     def get_mean_frequency(self):
         """
-        Computes the simplified frequency content characterisation parameter according to  Schnabel [1973]
+        Compute the simplified frequency content characterisation parameter according to  Schnabel [1973].
 
         Returns
         -------
@@ -194,7 +222,7 @@ class TimeSeries:
 
     def get_epsilon(self):
         """
-        Computes the dimensionless frequency indicator, epsilon as given by Clough and Penzien
+        Compute the dimensionless frequency indicator, epsilon as given by Clough and Penzien.
 
         Returns
         -------
@@ -214,14 +242,14 @@ class TimeSeries:
 
     def get_arias_intensity(self, g=False):
         """
-        Computes arias intensity:
+        Compute arias intensity.
 
         Parameters
         ----------
         g: Bool, optional
             g=True multiplies acceleration values with g=9.81 m/sec^2.
             Used when acceleration values in 'g' units are to be converted into 'm/sec^2'
-        
+
         Returns
         -------
         array like:
@@ -238,6 +266,7 @@ class TimeSeries:
 
     def get_total_arias(self, g=False):
         """
+        Calculate total arias intensity of the given signal.
 
         Parameters
         ----------
@@ -261,8 +290,7 @@ class TimeSeries:
 
     def get_sig_duration(self, g=False, start=0.05, stop=0.95):
         """
-
-        Computes significant duration as portion of ground motion encompassing 5% to 95% of total arias intensity
+        Compute significant duration as portion of ground motion encompassing 5% to 95% of total arias intensity.
 
         Parameters
         ----------
@@ -289,14 +317,14 @@ class TimeSeries:
 
     def get_destructive_potential(self, g=False):
         """
-        Computes destructiveness potential according to Araya and Sargoni (1984)
+        Compute destructiveness potential according to Araya and Sargoni (1984).
 
         Parameters
         ----------
         g: Bool, optional
             g=True multiplies acceleration values with g=9.81 m/sec^2.
             Used when acceleration values in 'g' units are to be converted into 'm/sec^2'
-        
+
         Returns
         -------
         Scalar:
@@ -305,12 +333,12 @@ class TimeSeries:
         """
         acc = self.y
         ia = self.get_total_arias(g=True)
-        u0 = len(np.where(np.diff(np.sign(acc)))[0])/self.time
+        u0 = len(np.where(np.diff(np.sign(acc)))[0])/self.duration
         return ia / u0**2
 
     def get_cum_abs_vel(self, g=False):
         """
-        Computes cummulative absolute velocity
+        Compute cummulative absolute velocity.
 
         Parameters
         ----------
@@ -333,7 +361,7 @@ class TimeSeries:
 
     def get_cum_abs_disp(self):
         """
-        Computes Cummulative Absolute Displacement
+        Compute Cummulative Absolute Displacement.
 
         Note
             Please make sure to use velocity time series as input to compute cummulative absolute displacement.
@@ -341,7 +369,7 @@ class TimeSeries:
         Parameters
         ----------
         None
-        
+
         Returns
         -------
         Scalar:
@@ -355,8 +383,7 @@ class TimeSeries:
 
     def get_specific_energy(self):
         """
-
-        Computes specific energy density
+        Compute specific energy density.
 
         Note
             Please use velocity time series as input to compute specific energy density.
@@ -364,7 +391,7 @@ class TimeSeries:
         Parameters
         ----------
         None
-        
+
         Returns
         -------
         Scalar:
@@ -377,8 +404,7 @@ class TimeSeries:
 
     def get_rms(self, g=False):
         """
-
-        Root-mean-square value of acceleration/velocity/displacement time series
+        Root-mean-square value of acceleration/velocity/displacement time series.
 
         Parameters
         ----------
@@ -386,23 +412,19 @@ class TimeSeries:
             g=True multiplies acceleration values with g=9.81 m/sec^2.
             Used when acceleration values in 'g' units are to be converted into 'm/sec^2'
 
-        
         Returns
         -------
         Scalar:
             Root-mean-square value of time series values
-
-
         """
         val = self.y
-        total_time = self.time
+        total_time = self.duration
         dt = self.dt
         return np.sqrt(trapz(val**2, dx=dt) * total_time**-1)
 
     def get_diff(self, edge_order=1):
         """
-        Differentiate the time series with respect to 't' (default).
-        Uses numpy.gradient() internally.
+        Differentiate the time series with respect to 't' (default) using numpy.gradient() internally.
 
         Parameters
         ----------
@@ -416,8 +438,7 @@ class TimeSeries:
 
     def get_int(self, **kwargs):
         """
-        Integrate the time series with respect to 't' (default).
-        Uses numpy.trapz() internally.
+        Integrate the time series with respect to 't' (default) using numpy.trapz() internally.
 
         Parameters
         ----------
@@ -431,11 +452,37 @@ class TimeSeries:
         kwargs = {**defaultArgs, **kwargs}
         return np.trapz(self.y, **kwargs)
 
+    def get_numerical_int(self, init=0.0, **kwargs):
+        """
+        Integrate the time series numerically w.r.t. t and produce 1D array of integrated values as opposed to get_int which gives total area under the curve.
+
+        Parameters
+        ----------
+        init (scalar): Initial condition
+
+        returns
+        -------
+        1D numpy array
+        """
+        defaultArgs = {"x": self.t}
+        kwargs = {**defaultArgs, **kwargs}
+        y = self.y
+        x = kwargs["x"]
+        yshift = np.zeros(len(y))
+        yshift[0:-1] = y[1:]
+        xshift = np.zeros(len(x))
+        xshift[0:-1] = x[1:]
+        area_vec = 0.5 * (y[0:-1] + y[1:]) * (x[1:] - x[0:-1])
+        yi = np.concatenate((np.array([init]), area_vec))
+        return np.cumsum(yi)
+
+
+class ResponseSpectra:
+    """ResponseSpectrum class."""
 
-class ResponseSpectrum:
     def __init__(self, T, Sd, Sv, Sa):
         """
-        Class for storing response spectra
+        Class for storing response spectrum.
 
         Parameters
         ----------
@@ -444,10 +491,12 @@ class ResponseSpectrum:
         Sv: (array of float) Spectral velocity
         Sa: (array of float) Spectral acceleration
         """
-        self.T = T
-        self.Sd = Sd
-        self.Sv = Sv
-        self.Sa = Sa
+        self.T = np.array(T)
+        self.Sd = np.array(Sd)
+        self.Sv = np.array(Sv)
+        self.Sa = np.array(Sa)
+        self.PSv = (2*np.pi / self.T) * self.Sd
+        self.PSa = (2*np.pi / self.T) * self.PSv
 
     def __repr__(self):
         a = ""
@@ -457,18 +506,21 @@ class ResponseSpectrum:
 
     def plot(self, log=False, **kwargs):
         """
-        Method plots Response Spectrum of ResponseSpectrum object
+        Plot Response Spectrum of ResponseSpectrum object.
+
         Parameters
         ----------
+        log (boolean): use log scale for axes (default *False*).
+
         Returns
         -------
         Matplotlib Figure Object
 
         """
-        fig, ax = plt.subplots(nrows=1, ncols=3, **kwargs)
-        ax[0].plot(self.T, self.Sd, color="black", linewidth=0.5)
-        ax[1].plot(self.T, self.Sv, color="black", linewidth=0.5)
-        ax[2].plot(self.T, self.Sa, color="black", linewidth=0.5)
+        fig, ax = plt.subplots(nrows=1, ncols=3, constrained_layout=True, **kwargs)
+        ax[0].plot(self.T, self.Sd)
+        ax[1].plot(self.T, self.Sv)
+        ax[2].plot(self.T, self.Sa)
         ax[0].set_xlabel("Period (s)")
         ax[1].set_xlabel("Period (s)")
         ax[2].set_xlabel("Period (s)")
@@ -487,9 +539,11 @@ class ResponseSpectrum:
 
 
 class FourierSpectrum:
+    """FourierSpectrum class."""
+
     def __init__(self, frequencies, ordinate, N):
         """
-        Class to store fourier spectra
+        Class to store fourier spectrum.
 
         Parameters
         ----------
@@ -514,16 +568,21 @@ class FourierSpectrum:
 
     def plot(self, log=False, **kwargs):
         """
-        Method plots Fourier Spectrum of FourierSpectrum object
+        Plot Fourier Spectrum of FourierSpectrum object.
 
         Parameters
         ----------
+        log (boolean): use log scale for axes (default *False*).
+
         Returns
         -------
         Matplotlib Figure Object
 
         """
+        defaultArgs = {"figsize": (10, 5)}
+        kwargs = {**defaultArgs, **kwargs}
         fig = plt.figure(constrained_layout=True, **kwargs)
+
         gs = plt.GridSpec(2, 2, figure=fig)
         ax0 = fig.add_subplot(gs[0, :])
         ax1 = fig.add_subplot(gs[1, 0])
@@ -534,16 +593,14 @@ class FourierSpectrum:
         ax0.plot(
             self.frequencies[:self.N//2],
             2.0 / self.N * self.amplitude[:self.N//2],
-            color="black",
-            linewidth=0.5,
         )
         ax1.set_xlabel("Frequency (Hz)")
         ax1.set_ylabel("Phase angle")
-        ax1.plot(self.frequencies[:self.N//2], self.phase[:self.N//2], color="black", linewidth=0.5)
+        ax1.plot(self.frequencies[:self.N//2], self.phase[:self.N//2])
         ax2.set_xlabel("Frequency (Hz)")
         ax2.set_ylabel("Unwrapped phase angle")
         ax2.plot(
-            np.sort(self.frequencies), self.unwrappedPhase, color="black", linewidth=0.5
+            np.sort(self.frequencies), self.unwrappedPhase
         )
         if log:
             ax0.set_xscale("log")
@@ -552,9 +609,11 @@ class FourierSpectrum:
 
 
 class PowerSpectrum:
+    """PowerSpectrum class."""
+
     def __init__(self, frequencies, amplitude, N):
         """
-        Class to store power spectra
+        Class to store power spectrum.
 
         Parameters
         ----------
@@ -565,15 +624,39 @@ class PowerSpectrum:
         self.amplitude = amplitude
         self.N = N
 
+        l0 = self.get_nth_moment(n=0)
+        l1 = self.get_nth_moment(n=1)
+        l2 = self.get_nth_moment(n=2)
+        self.centralFreq = np.sqrt(l2/l0)
+        self.shapeFactor = np.sqrt(1 - l1**2 / (l0*l2))
+
     def __repr__(self):
         a = ""
         for key, val in vars(self).items():
             a += "{:10s}:{}\n".format(key, val)
         return a
 
+    def get_nth_moment(self, n=0):
+        """
+        Calculate the nth moment of the power spectrum amplitude.
+        
+        Parameter:
+        ----------
+        n (scalar): order of moment
+
+        returns:
+        --------
+        nth moment
+        
+        """
+        N = self.N // 2
+        amp = self.amplitude[0:N]
+        freq = self.frequencies[0:N]
+        return np.trapz(freq**n * amp, x=freq)
+
     def get_compatible_timehistories(self, m=5):
         """
-        Generates m compatible timehistories from power spectrum
+        Generate m compatible timehistories from power spectrum.
 
         Parameters
         ----------
@@ -600,11 +683,11 @@ class PowerSpectrum:
 
         for k in range(m):
             if np.mod(n, 2) == 0:  # even number of samples
-                Xphase[1 : n // 2] = 2 * np.pi * np.random.rand(n // 2 - 1)
-                Xphase[n // 2 + 1 :] = -np.flip(Xphase[1 : n // 2])
+                Xphase[1: n // 2] = 2 * np.pi * np.random.rand(n // 2 - 1)
+                Xphase[n // 2 + 1:] = -np.flip(Xphase[1: n // 2])
             else:
-                Xphase[1 : (n + 1) // 2] = n * np.pi * np.random.rand((n + 1) // 2 - 1)
-                Xphase[(n + 1) // 2 :] = -np.flip(Xphase[1 : (n + 1) // 2])
+                Xphase[1: (n + 1) // 2] = n * np.pi * np.random.rand((n + 1) // 2 - 1)
+                Xphase[(n + 1) // 2:] = -np.flip(Xphase[1: (n + 1) // 2])
 
             X[:, k] = Xamp * (np.cos(Xphase) + 1j * np.sin(Xphase))
             x[:, k] = np.fft.ifft(X[:, k])
@@ -614,9 +697,11 @@ class PowerSpectrum:
 
     def plot(self, log=False):
         """
-        Method plots Power Spectrum of PowerSpectrum object
+        Plot Power Spectrum of PowerSpectrum object.
+
         Parameters
         ----------
+        log (boolean): use log scale for axes (default *False*).
         Returns
         -------
         Matplotlib Figure Object
@@ -626,9 +711,7 @@ class PowerSpectrum:
         ax.plot(
             self.frequencies,
             2.0 / self.N * self.amplitude,
-            color="black",
-            linewidth=0.5,
-        )
+            )
         ax.set_xlabel("Frequency (Hz)")
         ax.set_ylabel("Power Amplitude")
         if log:
